from base_st_classes import *

class PutfhSuite(NFSSuite):
    """Test operation 22: PUTFH

    FIXME: Add attribute directory and named attribute testing. 

    Equivalence partitioning:

    Input Condition: supplied filehandle
        Valid equivalence classes:
            file(1)
            dir(2)
            block(3)
            char(4)
            link(5)
            socket(6)
            FIFO(7)
            attribute directory(8)
            named attribute(9)
        Invalid equivalence classes:
            invalid filehandle(10)

    Comments: It's not possible to cover eq. class 10, since a filehandle
    is opaque to the client.
    """
    '''
    def setUp(self):
        NFSSuite.setUp(self)
        self.obj_name = "object1"

    def tearDown(self):
        try: self.clean_dir(self.tmp_dir)
        except SkipException, e:
            pass
    '''
    #
    # Testcases covering valid equivalence classes.
    #
    def testAllObjects(self):
        """PUTFH followed by GETFH on all type of objects

        Covered valid equivalence classes: 1, 2, 3, 4, 5, 6, 7
        """
        # Fetch filehandles of all types
        # List with (objpath, fh)
        filehandles = []
        for lookupops in map(self.ncl.lookup_path, self.all_objects):
            ops = [self.ncl.putrootfh_op()] + lookupops
            ops.append(self.ncl.getfh_op())
            res = self.ncl.do_ops(ops)
            self.assert_OK(res)

            fh = res.resarray[-1].arm.arm.object
            filehandles.append(fh)

        # Try PUTFH & GETFH on all these filehandles. 
        for (objpath, fh) in zip(self.all_objects, filehandles):
            putfhop = self.ncl.putfh_op(fh)
            getfhop = self.ncl.getfh_op()
            res = self.ncl.do_ops([putfhop, getfhop])
            self.assert_OK(res)

            new_fh = res.resarray[-1].arm.arm.object
            self.failIf(new_fh != fh, "GETFH after PUTFH returned different fh "\
                        "for object %s" % objpath)

    def testBadHandle(self):
        """PUTFH with bad filehandle should return NFS4ERR_BADHANDLE
        """
        putfhop = self.ncl.putfh_op("abc")
        res = self.ncl.do_ops([putfhop])
        self.assert_status(res,[NFS4ERR_BADHANDLE])
        
    def testStaleHandle(self):
        """PUTFH which nolonger exists should return NFS4ERR_STALE
        """
        # Create file
        self.init_connection()
        stale_fh,stateid = self.ncl.create_confirm()

        # Close file
        ops = [self.ncl.putfh_op(stale_fh)]
        ops.append(self.ncl.close_op(self.ncl.get_seqid(), stateid))
        res2 = self.ncl.do_ops(ops)
        self.assert_OK(res2)

        # Remove the file 
        ops = [self.ncl.putrootfh_op()] + self.ncl.lookup_path(self.tmp_dir)
        ops.append(self.ncl.remove_op(self.obj_name))
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)
        
        # Try to put the stale fh
        putfhop = self.ncl.putfh_op(stale_fh)
        res = self.ncl.do_ops([putfhop])
        self.assert_status(res,[NFS4ERR_STALE])
      
